/*
 * STM32L152 
 *      Flash:           0x08000000 - 0x0801FFFF (128 KB)
 *      Bootloader:      0x08000000 - 0x08003FFF (16 KB)
 *      Application:     0x08004000 - 0x0801FFFF (112 KB)
 */
#include <stdio.h>
#include <stdint.h>
#include <intrinsics.h>
#include "stm32l1xx.h"      // CMSIS Cortex-M3 Device 
#include <core_cm3.h>       // CMSIS Cortex-M3 Core

typedef void (application_t) (void);

typedef struct vector
{
    uint32_t        stack_addr;     // intvec[0] is initial Stack Pointer
    application_t   *func_p;        // intvec[1] is initial Program Counter
} vector_t;

extern const uint32_t app_vector;   // Application vector address symbol from 
                                    // the linker configuration file

// ------------------------------------------------------
int main(void)
{
    #pragma section = ".intvec_RAM"  // setting VTOR to __vector_RAM_table[]
    SCB->VTOR = (uint32_t)__section_begin(".intvec_RAM");  
    
    const vector_t *vector_p            = (vector_t*) &app_vector;
    volatile uint32_t stack_arr[100]    = {0}; // Allocate some stack
                                               // just to show that
                                               // the SP should be reset
                                               // before the jump - or the
                                               // stack won't be configured
                                               // correctly.
    __enable_interrupt();
    
    printf("Hello from bootloader!\n");
    
    __disable_interrupt();              // 1. Disable interrupts
    __set_SP(vector_p->stack_addr);     // 2. Configure stack pointer
    SCB->VTOR = (uint32_t) &app_vector; // 3. Configure VTOR
    vector_p->func_p();                 // 4. Jump to application
}

// ------------------------------------------------------
void SysTick_Handler(void)
{
    while (1) {}
}
